// file: dated_obs.cc 
// author: Bernt A Oedegaard
// class dated_observations:  

#include "dated_obs.h" 
#include <algorithm>

const double MISSING_OBS  = -1e40;
const double LOWEST_OBS   = -1e10;

//////////////////////////////////////////////////////////////////////
//  basic routines 

void dated_observations::set_title(string s) { title_ = s; };

void dated_observations::clear() { 
   title_ = string();
   dated<double>::clear();
};

/////////////////// describing and pulling //////////////////////

string dated_observations::title() const { return title_;};

int dated_observations::no_obs_between(const date& dat1, const date& dat2) const {
// count number of observations between given dates.
// not counting missing observations
   if (!dat1.valid()) return -1;
   if (!dat2.valid()) return -1;
   int noobs=0;
   int T = size();
   for (int t=0;t<T;++t) {
      if ( (dat1<=date_at(t) ) && (date_at(t)<=dat2) && (element_at(t)>LOWEST_OBS) ) ++noobs;
   };
   return noobs;
};

int dated_observations::no_obs_before(const date& d) const { 
   return no_obs_between(first_date(),d);
};

int dated_observations::no_obs_after(const date& d) const {
   return no_obs_between(d,last_date());
};

double dated_observations::max_obs(){
   vector<double> obs = elements();
   return *max_element(obs.begin(),obs.end()); 
};

double dated_observations::min_obs(){
   vector<double> obs = elements();
   return *min_element(obs.begin(),obs.end()); 
};

////////////////////////////////////////////////////////////////////////
/// output 

ostream& operator << (ostream& os, const dated_observations& obs) {
     // output list of observations
   if (obs.title().length()>0)   os <<  obs.title()  << endl; 
   for (int t=0;t<obs.size();t++) {
      os << " " << obs.date_at(t) << " " << obs.element_at(t) << endl;
   };
   return os;
}
