// file: dated_obs.h
// author: Bernt A Oedegaard.
// 
// dated Observations, a list of numbers with accompanying dates.
// inherit from dated<double>, and then additional functions

#ifndef _DATED_OBS_H_
#define _DATED_OBS_H_

#include <vector>     // vector templates
#include <string>     // ANSI string class
#include "date.h"       // my date class
#include "dated.h"      // templated dated<> class
#include "dated_util.h"      // utilities for templated dated<> class

class dated_observations : public dated<double>{
   private:
      string         title_;
   public:            
      ~dated_observations() { clear(); };
      void clear();                     // changing contents

      void set_title(string s); // title
      string title() const;

      int no_obs() const;
      int no_obs_between(const date& d1, const date& d2) const;
      int no_obs_before(const date& d) const;
      int no_obs_after(const date& d) const;

      double max_obs();
      double min_obs();
};

/////////////////////////// io /////////////////////////////
ostream& operator << (ostream&, const dated_observations& );

void print_dated_observations(ostream& of, 
			      dated_observations& d,
			      int precision=6);

///////////////////////// mischellaneous utilities ///////////////////

bool dates_match(const dated_observations& obs1,
		 const dated_observations& obs2);
void synchronize_dated_observations(dated_observations& obs1, 
				    dated_observations& obs2);   // fill with MISSING_OBS
dated_observations join(const dated_observations& obs1,const dated_observations& obs2);


/// differencing etc //////////////////////////////////////////////////////

dated_observations first_differences(const dated_observations& dobs);

dated_observations first_log_differences(const dated_observations& dobs);

//////////////////// Financial Calculations /////////////////////

dated_observations returns_given_prices(const dated_observations& prices); 

dated_observations wealth_given_returns(const date& first_date,
					const dated_observations& returns); 
// picking subsets.

dated_observations weekly_observations(const dated_observations&, int day_in_week);
dated_observations beginning_of_month_observations(const dated_observations&);
dated_observations end_of_month_observations(const dated_observations&);
dated_observations end_of_year_observations(const dated_observations&);
dated_observations end_of_year_current_observations(const dated_observations&);
dated_observations observations_between(const dated_observations& obs,const date& first,const date& last);
dated_observations observations_after(const dated_observations& obs, const date& first);
dated_observations observations_before(const dated_observations& obs, const date& last);
dated_observations observations_matching_dates(const dated_observations& obs, const vector<date>& dates);

#endif



