// file dobs_fin.cc
// author: Bernt Arne Odegaard
// dated observations: Financial calculations 

#include "dated_obs.h" 

const double MISSING_OBS = -1e40;
const double LOW_OBS = -1e10;

dated_observations returns_given_prices(const dated_observations& prices) {
    // assuming we are given prices, calculate simple returns 
    dated_observations rets; 
    rets.set_title(prices.title()+" : Returns");
    int T = prices.size();
    if (T<=2) return rets; 
    double P1 = prices.element_at(0);
    for (int t=1;t<T;++t){
	double P2 = prices.element_at(t);
	date d = prices.date_at(t);
	if  ( (P1>0.0) && (P2>0.0) ) {
	    double r = (P2-P1)/P1;
	    rets.append(d,r);
	}
	else { // append a missing observation.
	    rets.append(d,MISSING_OBS);
	};
	P1 = P2;
	d=prices.date_at(t);
    };
    return rets;
};

dated_observations wealth_given_returns(const date& first_date,
					const dated_observations& returns) 
{
    // assuming we are given raw returns, calculate the wealth relatives
    dated_observations wealth; 
    wealth.set_title(returns.title()+" wealth relatives ");
    wealth.append(first_date,1.0);
    int t=0;
    while(returns.date_at(t)<=first_date) {t++;};
    while(t<returns.size()) {
       if (returns.element_at(t)>LOW_OBS) {
	  wealth.append(returns.date_at(t),wealth.element_at(t)*(1+returns.element_at(t)));
       }
       else {
	  wealth.append(returns.date_at(t),MISSING_OBS);
       };
       t++;
    };
    return wealth;
};
