// file: stock_price_history_util.cc 
// author: Bernt A Oedegaard
// non member functions useful in the class for dealing with historical stock 
// returns.
//
//    get_returns_from_prices
//    returns
//    get prices
//    get return

#include "stock_price_history.h"

dated_observations returns_from_prices(const stock_price_history& st,
				       const dated_observations& prices) {
  if (prices.size()<=2){ return dated_observations(); };
  if (   (!st.dividends_between(prices.first_date(),prices.last_date())) 
	 && (!st.adjustments_between(prices.first_date(),
				     prices.last_date()) ) )  { 
     // nothing special, can use the standard return calculation in dated_obs
     return returns_given_prices(prices);
  };
  // else there are adjustments or dividends
  dated_observations rets;
  for (int i=1;i<prices.size();++i) {
     if ((prices.element_at(i)>0.0) && (prices.element_at(i-1)>0.0)) {
	double ret=-1e99;
	if (!st.adjustments_between(prices.date_at(i-1), prices.date_at(i) )) {
	      ret = ( prices.element_at(i) +
		      st.total_dividends_between(prices.date_at(i-1),prices.date_at(i))
		      - prices.element_at(i-1))/prices.element_at(i-1);
	}
	else {
	     ret  = ( prices.element_at(i)
		      /st.aggregated_adjustments_between(prices.date_at(i-1), 
							 prices.date_at(i))
		      + st.total_dividends_between(prices.date_at(i-1),
						   prices.date_at(i))
		      - prices.element_at(i-1) ) / 
		prices.element_at(i-1);
	};
	rets.append(prices.date_at(i),ret);
     };
  };
  return rets;
};


dated_observations returns_from_prices_without_dividends(const stock_price_history& st,
							 const dated_observations& prices) {
  if (prices.size()<=2){ return dated_observations(); };
  if (!st.adjustments_between(prices.first_date(),prices.last_date()))  {
     // nothing special, can use the standard return calculation in dated_obs
     return returns_given_prices(prices);
  };
  // else there are adjustments or dividends
  dated_observations rets;
  for (int i=1;i<prices.size();++i) {
     if ((prices.element_at(i)>0.0) && (prices.element_at(i-1)>0.0)) {
	double ret=-1e99;
	if (!st.adjustments_between(prices.date_at(i-1), prices.date_at(i) )) {
	      ret = ( prices.element_at(i) 
//		      +st.total_dividends_between(prices.date_at(i-1),prices.date_at(i))
		      - prices.element_at(i-1))/prices.element_at(i-1);
	}
	else {
	     ret  = ( prices.element_at(i)
		      /st.aggregated_adjustments_between(prices.date_at(i-1), 
							 prices.date_at(i))
//		      + st.total_dividends_between(prices.date_at(i-1), prices.date_at(i))
		      - prices.element_at(i-1) ) / 
		prices.element_at(i-1);
	};
	rets.append(prices.date_at(i),ret);
     };
  };
  return rets;
};

dated_observations bid_ask_returns(const stock_price_history& st) {
   dated_observations rets;
   if (st.no_dates()<=2){ return rets; };
   if (   (!st.dividends_between(st.first_date(),st.last_date()))  // save a lot of testing in loop
	  && (!st.adjustments_between(st.first_date(),st.last_date())) )  {
      for (unsigned int i=1;i<st.no_prices();++i) { 
	 if ((st.SellPrice(i)>0.0) && (st.BuyPrice(i-1)>0.0)) {
	    rets.append( st.date_at(i),
			 ( (st.SellPrice(i)-st.BuyPrice(i-1)) / st.BuyPrice(i-1) ) );
	 };
      };
   }
   else {
      for (unsigned i=1;i<st.no_prices();++i) {
	 if ((st.SellPrice(i)>0.0) && (st.BuyPrice(i-1)>0.0)) {
	    if (!st.adjustments_between(st.date_at(i-1),st.date_at(i))) {
		  rets.append(st.date_at(i),
			      ( st.SellPrice(i)
				+ st.total_dividends_between(st.date_at(i-1), st.date_at(i))
				- st.BuyPrice(i-1))
			      /st.BuyPrice(i-1));
	    }
	    else {
	       rets.append(st.date_at(i),
			   (st.SellPrice(i)/st.aggregated_adjustments_between(st.date_at(i-1), st.date_at(i))
			    + st.total_dividends_between(st.date_at(i-1),st.date_at(i))
			    -st.BuyPrice(i-1))
			   / st.BuyPrice(i-1));
	    };
	 };
      };
   };
   return rets;
};

////////////////////// pulling prices /////////////////////////////////////

dated_observations daily_prices(const stock_price_history& stock){ 
   dated_observations dprices;
   for (unsigned int t=0; t<stock.no_prices();++t){
      if (stock.price(t)>0) {
	 dprices.append(stock.date_at(t),stock.price(t));
      };
   };
   return dprices;
}; 

dated_observations daily_trade_prices(const stock_price_history& stock) {
   dated_observations dpric;
   for (unsigned int t=0; t<stock.no_prices();++t){
      if  (stock.trade(t)>0) {
	 dpric.append(stock.date_at(t),stock.trade(t));
      };
   };
   return dpric;
}; 

dated_observations monthly_prices(const stock_price_history& stock) {
   dated_observations dpric = daily_prices(stock);
   return end_of_month_observations(dpric);
};

dated_observations weekly_prices(const stock_price_history& stock, int day_of_week) {
   dated_observations dpric = daily_prices(stock);
   return weekly_observations(dpric,day_of_week);
};

dated_observations annual_prices(const stock_price_history& stock) {
   dated_observations dpric = daily_prices(stock);
   return end_of_year_observations(dpric);
};

//////////////////////////////////// pulling returns ///////////////////////////////

dated_observations daily_returns(const stock_price_history& stock) {
   dated_observations dpric = daily_prices(stock);
   return returns_from_prices(stock,dpric);
};

dated_observations daily_returns_without_dividends(const stock_price_history& stock) {
   dated_observations dpric = daily_prices(stock);
   return returns_from_prices_without_dividends(stock,dpric);
};

dated_observations daily_returns(const stock_price_history& stock,
				 date from,
				 date to) {
   dated_observations tmp_ret = daily_returns(stock);
   return observations_between(tmp_ret,from,to);
};


dated_observations daily_trade_returns(const stock_price_history& stock) {
   dated_observations dpric = daily_trade_prices(stock);
   return returns_from_prices(stock,dpric);
};

dated_observations daily_trade_returns(const stock_price_history& stock, 
				       date from, date to) {
   return observations_between(daily_trade_returns(stock),from,to);
};

dated_observations weekly_returns(const stock_price_history& stock, int day_of_week) {
   dated_observations wpric = weekly_prices(stock,day_of_week);
   return returns_from_prices(stock,wpric);
};

dated_observations monthly_returns(const stock_price_history& stock) {
   dated_observations mpric = monthly_prices(stock);
   return returns_from_prices(stock,mpric);
};

dated_observations annual_returns(const stock_price_history& stock) {
   dated_observations apric = annual_prices(stock);
   return returns_from_prices(stock, apric);
};
