// file: tst_dated.cc

#include "dated.h"
#include <string>

void tst_simple_dated(){
   cout << "TESTING simple " << endl;
   dated<double> dd;
   dd.insert(date(1,1,1998),1);
   dd.insert(date(1,2,1998),2);
   cout << dd.size() << " elements " << endl;
   cout << dd ;
   cout << " inserting at 1/1 and 1/3 " << endl;
   dd.insert(date(1,1,1998),0.5);
   dd.insert(date(1,3,1998),3);
   cout << dd;
   cout << " deleting 1/2" << endl;
   dd.remove(date(1,3,1998));
   cout << dd;
   cout << " dated<string> " << endl;
   dated<string> ds;
   ds.insert(date(1,1,1998),"A");
   ds.insert(date(1,2,1998),"B");
   cout << ds;
   cout << "DONE simple test"  << endl;
};

dated<double> create_test_series_of_dated(int no_elements=5) {
   dated<double> dd;
   date d(1,1,1998);
   for (int t=1; t<=no_elements; ++t) {
      dd.insert(d++,t);
   };
   return dd;
};

void tst_copying_dated () {
   cout << "TESTING copying " << endl;
   dated<double> dd = create_test_series_of_dated();
   cout << " original " << endl;
   cout << dd;
   cout << " copy " << endl;
   dated<double> dd2 = dd;
   cout << dd2;
   cout << "DONE TESTING copying " << endl;
};

void tst_pull_dated(){
   cout << "TESTING pulling elements " << endl;
   dated<double> dd = create_test_series_of_dated();
   vector<date> ds = dd.dates();
   vector<double> dobs = dd.elements();
   cout << "DONE TESTING pulling elements " << endl;
};

void tst_deletion_dated(){
   cout << "TESTING deletion of dated " << endl;
   dated<double> dd = create_test_series_of_dated(12);
   cout << dd ;
   cout << " deleting between " << dd.date_at(3) << dd.date_at(6) << endl;
   dd.remove_between(dd.date_at(3),dd.date_at(6));
   cout << dd ;
   cout << " deleting before " << dd.date_at(3);
   dd.remove_before(dd.date_at(3));
   cout << dd ;
   cout << " deleting after " << dd.date_at(3);
   dd.remove_after(dd.date_at(3));
   cout << dd ;
   cout << "DONE TESTING deletions " << endl;
};

void tst_current_element(){
   cout << "TESTING current element " << endl;
   dated<double> dd = create_test_series_of_dated(3);
   dd.remove(dd.date_at(1));
   cout << " dated obs " << endl;
   cout << dd << endl;
   date d=dd.date_at(0);
   --d;
   for (int i=0;i<5;++i) {
      cout << " current element at " << d << " " << dd.current_element_at(d) << endl;
      d++;
   };
   cout << "DONE TESTING current element " << endl;
};

/* // to test this need to remove private qualification 
void tst_indexing(){
   cout << "TESTING indexing " << endl;
   dated<double> dd = create_test_series_of_dated(3);
   vector<date> dates = dd.dates();
   dd.remove(dd.date_at(1));
   cout << " dated obs " << endl;
   cout << dd << endl;
   date d;

   d=dd.date_at(0);
   --d;
   for (int i=0;i<5;++i) {
      cout << " index of date " << d << " " << dd.index_of_date(d) << endl;
      d++;
   };

   d=dd.date_at(0);
   --d;
   for (int i=0;i<5;++i) {
      cout << " index of last date before " << d 
	   << " " << dd.index_of_last_date_before(d) << endl;
      d++;
   };


   d=dd.date_at(0);
   --d;
   for (int i=0;i<5;++i) {
      cout << " index of first date after " << d 
	   << " " << dd.index_of_first_date_after(d) << endl;
      d++;
   };

   cout << "DONE TESTING indexing " << endl;
};
*/
void test_dated(){
   cout << "TESTING DATED " << endl;
   tst_simple_dated();
   tst_copying_dated();
   tst_pull_dated();
   tst_deletion_dated();
   tst_current_element();
   //   tst_indexing();
   cout << "DONE TESTING DATED " << endl;
};

