// file tst_stock_price_history.cc  
// author Bernt A Oedegaard
// test historical observations of stocks.

#include "stock_price_history.h"

//////////////////////////////////////////////////////
///  simplest possible test //////////////////////////
//////////////////////////////////////////////////////

void tst_simple(){
   cout << "TESTING simple creation and copying of stock price history " << endl;
   stock_price_history sh;
   sh.set_security_name("Stock name");
   date d(1,1,1996);
   double bid   = 99.8;
   double price = 100;
   double ask   = 100.8;
   sh.add_prices(d++,bid,ask,price);
   double e = 0.04;
   sh.add_prices(d++,bid+=e,ask+=e,price+=e);
   e=-0.2;
   sh.add_prices(d++,bid+=e,ask+=e,price+=e); 
   
   dump(cout,sh);
   stock_price_history sh1(sh);
   cout << "dumping constr copy " << endl;
   dump(cout,sh1);
   stock_price_history sh2 = sh; 
   cout << " dumping copy" << endl;
   dump(cout,sh2);
   sh2.clear();
   sh2=sh;
   cout << " dumping copied onto copy" << endl;
   dump(cout,sh2);
   cout << " done test copying " << endl;
   cout << "DONE TESTING simple creation and copying of stock price history " << endl;
};

////////////////////////////////////////////////////////////////////
// general utility used in other testing. //////////////////////////
////////////////////////////////////////////////////////////////////

stock_price_history create_test_stock_history(int no_days=20){
   stock_price_history sh;
   date d(1,1,1980);
   double p=1;
   for (int day=0;day<no_days;day++) {
      sh.add_prices(d,p-0.005,p,p+0.005);
      d++;
      p*=1.01;
   };
   return sh;
}

/////////////////////////////////////////////////////////////
// test basics ////////
/////////////////////////////////////////////////////////////

void tst_empty(){
    cout << "TEST: whether empty works well " << endl;
    stock_price_history sh;
    if (sh.empty()) { cout << " claims empty when empty " << endl; }
    else { cout << " claims nonempty when empty " << endl; }
    sh.set_security_name("name");
    if (sh.empty()) { cout << " claims empty when nonempty " << endl; }
    else { cout << " claims nonempty when nonempty " << endl; }
    sh.set_security_name("");
    if (sh.empty()) { cout << " claims empty when empty " << endl; }
    else { cout << " claims nonempty when empty " << endl; }
    sh.add_dividend(date(1,1,1980),10.0);
    if (sh.empty()) { cout << " claims empty when nonempty " << endl; }
    else { cout << " claims nonempty when nonempty " << endl; }
    cout << "DONE test of empty " << endl;
};

void tst_delete() {
   cout << "TESTING: Delete prices " << endl;
   cout << " first create stock price history with 10 obs " << endl;
   stock_price_history sh = create_test_stock_history(10);
   cout << " before doing any erasing " << endl;
   dump(cout,sh);
   sh.erase_before(date(4,1,80));
   cout << "after deleting  before 4/1" << endl;
   dump(cout,sh);
   sh.erase_after(date(7,1,80));
   cout << "after deleting after 7/1" << endl;
   dump(cout,sh); 
   cout << "DONE testing deleting " << endl;
};

void tst_basics(){
   tst_empty();
   tst_delete();
};

///////////////////////////////////////////////////////////////////
// testing adjustments ////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

void tst_adjustments(){
   cout << "TESTING adding adjustments to stock price history " << endl;
   cout << " first create 2 period " << endl;
   stock_price_history sh = create_test_stock_history(2);
   dump(cout,sh);
   cout << " then add adjustment on 1/1/80" << endl;
   sh.add_adjustment(date(1,1,1980),1.5);
   dump(cout,sh);
   cout << " then remove adjustment on 1/1/80" << endl;
   sh.remove_adjustment(date(1,1,1980));
   dump(cout,sh);
   cout << "DONE TESTING adjustments " << endl;
};


///////////////////////////////////////////////////////////////////
// testing dividends ////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

void tst_dividends(){
   cout << "TESTING adding dividends to stock price history " << endl;
   cout << " first create 2 period stock price history " << endl;
   stock_price_history sh = create_test_stock_history(2);
   dump(cout,sh);
   cout << " then add dividend on 1/1/80" << endl;
   sh.add_dividend(date(1,1,1980),1.5);
   dump(cout,sh);
   cout << " then remove dividend on 1/1/80" << endl;
   sh.remove_dividend(date(1,1,1980));
   dump(cout,sh);
   cout << "DONE TESTING dividends " << endl;
};


void tst_stock_price_history(){
    cout << "TESTING: stock price history class " << endl;
//    tst_simple();
//    tst_basics();
//    tst_stock_specials();
//    tst_returns();
//    tst_io();
    tst_adjustments();
    tst_dividends();
    cout << "DONE: all testing of stock price history class " << endl;
};
